@@ -38,7 +38,6 @@ ImageView::ImageView(QWidget *parent, MetadataCache *mdCache) : QWidget(parent)
 	imageLabel->setScaledContents(true);
 	isAnimation = false;
 	anim = 0;
-	setPalette(QPalette(GData::backgroundColor));
 
 	QHBoxLayout *mainLayout = new QHBoxLayout();
 	mainLayout->setContentsMargins(0, 0, 0, 0);
@@ -55,6 +54,7 @@ ImageView::ImageView(QWidget *parent, MetadataCache *mdCache) : QWidget(parent)
 	scrlArea->setFrameStyle(0);
 	scrlArea->setLayout(mainLayout);
 	scrlArea->setWidgetResizable(true);
+	setBgColor();
 
 	QVBoxLayout *scrollLayout = new QVBoxLayout;
 	scrollLayout->setContentsMargins(0, 0, 0, 0);
@@ -134,11 +134,11 @@ void ImageView::resizeImage()
 	QSize imgSize = isAnimation? anim->currentPixmap().size() : imageLabel->pixmap()->size();
 
 	if (tempDisableResize) {
-		imgSize.scale(calcZoom(imgSize.width()), calcZoom(imgSize.height()), Qt::KeepAspectRatio);
+		imgSize.scale(imgSize.width(), imgSize.height(), Qt::KeepAspectRatio);
 	} else {
 		switch(GData::zoomInFlags) {
 			case Disable:
-				if (imgSize.width() < imageViewWidth && imgSize.height() < imageViewHeight) {
+				if (imgSize.width() <= imageViewWidth && imgSize.height() <= imageViewHeight) {
 					imgSize.scale(	calcZoom(imgSize.width()),
 									calcZoom(imgSize.height()),
 									Qt::KeepAspectRatio);
@@ -146,7 +146,7 @@ void ImageView::resizeImage()
 				break;
 				
 			case WidthNHeight:
-				if (imgSize.width() < imageViewWidth && imgSize.height() < imageViewHeight) {
+				if (imgSize.width() <= imageViewWidth && imgSize.height() <= imageViewHeight) {
 					imgSize.scale(	calcZoom(imageViewWidth),
 									calcZoom(imageViewHeight),
 									Qt::KeepAspectRatio);
@@ -154,7 +154,7 @@ void ImageView::resizeImage()
 				break;
 
 			case Width:
-				if (imgSize.width() < imageViewWidth) {
+				if (imgSize.width() <= imageViewWidth) {
 					imgSize.scale(	calcZoom(imageViewWidth), 
 									calcZoom(getHeightByWidth(	imgSize.width(),
 																imgSize.height(),
@@ -164,7 +164,7 @@ void ImageView::resizeImage()
 				break;
 				
 			case Height:
-				if (imgSize.height() < imageViewHeight) {
+				if (imgSize.height() <= imageViewHeight) {
 					imgSize.scale(	calcZoom(getWidthByHeight(	imgSize.height(), 
 																imgSize.width(),
 																imageViewHeight)),
@@ -175,10 +175,10 @@ void ImageView::resizeImage()
 
 			case Disprop:
 				int newWidth = imgSize.width(), newHeight = imgSize.height();
-				if (newWidth < imageViewWidth) {
+				if (newWidth <= imageViewWidth) {
 					newWidth = imageViewWidth;
 				}
-				if (newHeight < imageViewHeight) {
+				if (newHeight <= imageViewHeight) {
 					newHeight = imageViewHeight;
 				}
 				imgSize.scale(calcZoom(newWidth), calcZoom(newHeight), Qt::IgnoreAspectRatio);
@@ -203,7 +203,7 @@ void ImageView::resizeImage()
 				break;
 
 			case Width:
-				if (imgSize.width() > imageViewWidth) {
+				if (imgSize.width() >= imageViewWidth) {
 					imgSize.scale(	calcZoom(imageViewWidth), 
 									calcZoom(getHeightByWidth(	imgSize.width(), 
 																imgSize.height(), 
@@ -213,7 +213,7 @@ void ImageView::resizeImage()
 				break;
 				
 			case Height:
-				if (imgSize.height() > imageViewHeight) {
+				if (imgSize.height() >= imageViewHeight) {
 					imgSize.scale(	calcZoom(getWidthByHeight(	imgSize.height(),
 																imgSize.width(), 
 																imageViewHeight)),
@@ -224,10 +224,10 @@ void ImageView::resizeImage()
 
 			case Disprop:
 				int newWidth = imgSize.width(), newHeight = imgSize.height();
-				if (newWidth > imageViewWidth) {
+				if (newWidth >= imageViewWidth) {
 					newWidth = imageViewWidth;
 				}
-				if (newHeight > imageViewHeight) {
+				if (newHeight >= imageViewHeight) {
 					newHeight = imageViewHeight;
 				}
 				imgSize.scale(calcZoom(newWidth), calcZoom(newHeight), Qt::IgnoreAspectRatio);
@@ -851,28 +851,31 @@ void ImageView::mouseMoveEvent(QMouseEvent *event)
 			int newY = layoutY + (event->pos().y() - mouseY);
 			bool needToMove = false;
 
-			if (imageLabel->size().width() > size().width()) 	{
-				if (newX > 0)
+			if (imageLabel->size().width() > size().width()) {
+				if (newX > 0) {
 					newX = 0;
-				else if (newX < (size().width() - imageLabel->size().width()))
+				} else if (newX < (size().width() - imageLabel->size().width())) {
 					newX = (size().width() - imageLabel->size().width());
+				}
 				needToMove = true;
-			}
-			else
+			} else {
 				newX = layoutX;
+			}
 
 			if (imageLabel->size().height() > size().height()) {
-				if (newY > 0)
+				if (newY > 0) {
 					newY = 0;
-				else if (newY < (size().height() - imageLabel->size().height()))
+				} else if (newY < (size().height() - imageLabel->size().height())) {
 					newY = (size().height() - imageLabel->size().height());
+				}
 				needToMove = true;
-			}
-			else
+			} else {
 				newY = layoutY;
+			}
 
-			if (needToMove)
+			if (needToMove) {
 				imageLabel->move(newX, newY);
+			}
 		}
 	}
 }
@@ -1078,3 +1081,14 @@ void ImageView::pasteImage()
 	}
 }
 
+void ImageView::setBgColor()
+{
+	QString bgColor = "background: rgb(%1, %2, %3); ";
+	bgColor = bgColor.arg(GData::backgroundColor.red())
+						.arg(GData::backgroundColor.green())
+						.arg(GData::backgroundColor.blue());
+
+	QString ss = "QWidget { " + bgColor + " }";
+	scrlArea->setStyleSheet(ss);
+}
+
