@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013 Ofer Kashayov <oferkv@live.com>
+ *  Copyright (C) 2013-2014 Ofer Kashayov <oferkv@live.com>
  *  This file is part of Phototonic Image Viewer.
  *
  *  Phototonic is free software: you can redistribute it and/or modify
@@ -18,10 +18,9 @@
 
 #include "thumbview.h"
 
-#define BAD_IMG_SZ	64
-
-ThumbView::ThumbView(QWidget *parent) : QListView(parent)
+ThumbView::ThumbView(QWidget *parent, MetadataCache *mdCache) : QListView(parent)
 {
+	this->mdCache = mdCache;
 	GData::thumbsBackgroundColor = GData::appSettings->value("backgroundThumbColor").value<QColor>();
 	GData::thumbsTextColor = GData::appSettings->value("textThumbColor").value<QColor>();
 	setThumbColors();
@@ -65,9 +64,18 @@ ThumbView::ThumbView(QWidget *parent) : QListView(parent)
 
 void ThumbView::setThumbColors()
 {
+	QString bgColor = "background: rgb(%1, %2, %3); ";
+	bgColor = bgColor.arg(GData::thumbsBackgroundColor.red())
+						.arg(GData::thumbsBackgroundColor.green())
+						.arg(GData::thumbsBackgroundColor.blue());
+
+	QString ss = "QListView { " + bgColor + "background-image: url("
+					+ GData::thumbsBackImage
+					+ "); background-attachment: fixed; }";
+   	setStyleSheet(ss);
+
 	QPalette scrollBarOrigPal = verticalScrollBar()->palette();
 	QPalette thumbViewOrigPal = palette();
-	thumbViewOrigPal.setColor(QPalette::Base, GData::thumbsBackgroundColor);
 	thumbViewOrigPal.setColor(QPalette::Text, GData::thumbsTextColor);
 	setPalette(thumbViewOrigPal);
 	verticalScrollBar()->setPalette(scrollBarOrigPal);
@@ -175,24 +183,19 @@ void ThumbView::updateExifInfo(QString imageFullPath)
 	QString key;
 	QString val;
 
-	try
-	{
+	try	{
 		exifImage = Exiv2::ImageFactory::open(imageFullPath.toStdString());
+		exifImage->readMetadata();
 	}
-	catch (Exiv2::Error &error)
-	{
+	catch (Exiv2::Error &error) {
 		return;
 	}
 
-	exifImage->readMetadata();
 	Exiv2::ExifData &exifData = exifImage->exifData();
-
-	if (!exifData.empty())
-	{
+	if (!exifData.empty()) {
 		Exiv2::ExifData::const_iterator end = exifData.end();
-		for (Exiv2::ExifData::const_iterator md = exifData.begin(); md != end; ++md)
-		{
-			// qDebug() << Exiv2::toString(md->key()).c_str() << " " << Exiv2::toString(md->value()).c_str();
+		infoView->addTitleEntry("Exif");
+		for (Exiv2::ExifData::const_iterator md = exifData.begin(); md != end; ++md) {
 			key = QString::fromUtf8(md->tagName().c_str());
 			val = QString::fromUtf8(md->print().c_str());
 			infoView->addEntry(key, val);
@@ -200,11 +203,10 @@ void ThumbView::updateExifInfo(QString imageFullPath)
 	}
 
 	Exiv2::IptcData &iptcData = exifImage->iptcData();
-	if (!iptcData.empty())
-	{
+	if (!iptcData.empty()) {
 		Exiv2::IptcData::iterator end = iptcData.end();
-		for (Exiv2::IptcData::iterator md = iptcData.begin(); md != end; ++md)
-		{
+		infoView->addTitleEntry("IPTC");
+		for (Exiv2::IptcData::iterator md = iptcData.begin(); md != end; ++md) {
 			key = QString::fromUtf8(md->tagName().c_str());
 			val = QString::fromUtf8(md->print().c_str());
 	   		infoView->addEntry(key, val);
@@ -212,11 +214,10 @@ void ThumbView::updateExifInfo(QString imageFullPath)
 	}
 
 	Exiv2::XmpData &xmpData = exifImage->xmpData();
-	if (!xmpData.empty())
-	{
+	if (!xmpData.empty()) {
 		Exiv2::XmpData::iterator end = xmpData.end();
-		for (Exiv2::XmpData::iterator md = xmpData.begin(); md != end; ++md)
-		{
+		infoView->addTitleEntry("XMP");
+		for (Exiv2::XmpData::iterator md = xmpData.begin(); md != end; ++md) {
 			key = QString::fromUtf8(md->tagName().c_str());
 			val = QString::fromUtf8(md->print().c_str());
 			infoView->addEntry(key, val);
@@ -230,35 +231,43 @@ void ThumbView::handleSelectionChanged(const QItemSelection&)
 	QModelIndexList indexesList = selectionModel()->selectedIndexes();
 	int nSelected = indexesList.size();
 	QString imageFullPath;
+	QString statusStr;
 
 	infoView->clear();
 
-	if (nSelected == 1)
-	{
+	if (nSelected == 1) {
 		QString imageFullPath = thumbViewModel->item(indexesList.first().row())->data(FileNameRole).toString();
 		imageInfoReader.setFileName(imageFullPath);
 		QString key;
 		QString val;
 
-		if (imageInfoReader.size().isValid())
-		{
-			QFileInfo imageInfo = QFileInfo(imageFullPath);
+		QFileInfo imageInfo = QFileInfo(imageFullPath);
+		infoView->addTitleEntry(tr("General"));
 
-			key = tr("File name");
-			val = imageInfo.fileName();
-			infoView->addEntry(key, val);
+		key = tr("File name");
+		val = imageInfo.fileName();
+		infoView->addEntry(key, val);
 
-			key = tr("Location");
-			val = imageInfo.path();
-			infoView->addEntry(key, val);
+		key = tr("Location");
+		val = imageInfo.path();
+		infoView->addEntry(key, val);
 
+		key = tr("Size");
+		val = QString::number(imageInfo.size() / 1024.0, 'f', 2) + "K";
+		infoView->addEntry(key, val);
+
+		key = tr("Modified");
+		val = imageInfo.lastModified().toString(Qt::SystemLocaleShortDate);
+		infoView->addEntry(key, val);
+
+		if (imageInfoReader.size().isValid()) {
 			key = tr("Format");
 			val = imageInfoReader.format().toUpper();
 			infoView->addEntry(key, val);
 
 			key = tr("Resolution");
 			val = QString::number(imageInfoReader.size().width())
-					+ " x "
+					+ "x"
 					+ QString::number(imageInfoReader.size().height());
 			infoView->addEntry(key, val);
 
@@ -266,40 +275,93 @@ void ThumbView::handleSelectionChanged(const QItemSelection&)
 			val = QString::number((imageInfoReader.size().width() * imageInfoReader.size().height()) / 1000000.0, 'f', 2);
 			infoView->addEntry(key, val);
 
-			key = tr("Size");
-			val = QString::number(imageInfo.size() / 1024.0, 'f', 2) + "K";
-			infoView->addEntry(key, val);
-
-			key = tr("Modified");
-			val = imageInfo.lastModified().toString(Qt::SystemLocaleShortDate);
-			infoView->addEntry(key, val);
-			
 			updateExifInfo(imageFullPath);
 			recentThumb = imageFullPath;
-		}
-		else
-		{
+		} else {
 			imageInfoReader.read();
-			key = imageInfoReader.errorString();
-			val = "";
+			key = tr("Error");
+			val = imageInfoReader.errorString();
 			infoView->addEntry(key, val);
 		}
 	}
-	updateThumbsSelection();
+
+	if (imageTags->currentDisplayMode == SelectionTagsDisplay) {
+		imageTags->showSelectedImagesTags();
+	}
+
+	/* update status bar */
+	if (!nSelected) {
+		updateThumbsCount();
+		return;
+	} else if (nSelected >= 1) {
+		statusStr = tr("Selected %1 of%2")
+					.arg(QString::number(nSelected))
+					.arg(tr(" %n image(s)", "", thumbViewModel->rowCount()));
+	}
+
+	emit setStatus(statusStr);
+}
+
+QStringList ThumbView::getSelectedThumbsList()
+{
+	QModelIndexList indexesList = selectionModel()->selectedIndexes();
+	QStringList SelectedThumbsPaths;
+	
+	for (int tn = indexesList.size() - 1; tn >= 0 ; --tn) {
+		SelectedThumbsPaths << 
+			thumbViewModel->item(indexesList[tn].row())->data(FileNameRole).toString();
+	}
+
+	return SelectedThumbsPaths;
 }
 
 void ThumbView::startDrag(Qt::DropActions)
 {
 	QModelIndexList indexesList = selectionModel()->selectedIndexes();
-	if (indexesList.isEmpty())
-	{
+	if (indexesList.isEmpty()) {
 		return;
 	}
 
 	QDrag *drag = new QDrag(this);
 	QMimeData *mimeData = new QMimeData;
+	QList<QUrl> urls;
+	for (QModelIndexList::const_iterator it = indexesList.constBegin(),
+										end = indexesList.constEnd(); it != end; ++it)
+	{
+		urls << QUrl(thumbViewModel->item(it->row())->data(FileNameRole).toString());
+	}
+	mimeData->setUrls(urls);
 	drag->setMimeData(mimeData);
-	drag->start(Qt::CopyAction | Qt::MoveAction);
+	QPixmap pix;
+	if (indexesList.count() > 1) {
+		pix = QPixmap(128, 112);
+		pix.fill(Qt::transparent);
+		QPainter painter(&pix);
+		painter.setBrush(Qt::NoBrush);
+		painter.setPen(QPen(Qt::white, 2));
+		int x = 0, y = 0, xMax = 0, yMax = 0;
+		for (int i = 0; i < qMin(5, indexesList.count()); ++i) {
+			QPixmap pix = thumbViewModel->item(indexesList.at(i).row())->icon().pixmap(72);
+			if (i == 4) {
+				x = (xMax - pix.width()) / 2;
+				y = (yMax - pix.height()) / 2;
+			}
+			painter.drawPixmap(x, y, pix);
+			xMax = qMax(xMax, qMin(128, x + pix.width()));
+			yMax = qMax(yMax, qMin(112, y + pix.height()));
+			painter.drawRect(x + 1, y + 1, qMin(126, pix.width() - 2), qMin(110, pix.height() - 2));
+			x = !(x == y) * 56;
+			y = !y * 40;
+		}
+		painter.end();
+		pix = pix.copy(0, 0, xMax, yMax);
+		drag->setPixmap(pix);
+	} else {
+		pix = thumbViewModel->item(indexesList.at(0).row())->icon().pixmap(128);
+		drag->setPixmap(pix);
+	}
+	drag->setHotSpot(QPoint(pix.width() / 2, pix.height() / 2));
+	drag->exec(Qt::CopyAction | Qt::MoveAction | Qt::LinkAction, Qt::IgnoreAction);
 }
 
 void ThumbView::abort()
@@ -310,54 +372,59 @@ void ThumbView::abort()
 void ThumbView::loadVisibleThumbs(int scrollBarValue)
 {
 	static int lastScrollBarValue = 0;
-	scrolledForward = (scrollBarValue >= lastScrollBarValue);
+
+	if (GData::thumbsLayout == Compact) {
+		scrolledForward = true;
+	} else {
+		scrolledForward = (scrollBarValue >= lastScrollBarValue);
+	}
+
 	lastScrollBarValue = scrollBarValue;
 
 Start:
-
-	int first = getFirstVisibleThumb();
-	int last = getLastVisibleThumb();
-	if (abortOp || first < 0 || last < 0) 
-		return;
-
-	if (scrolledForward)
-	{
-		last += ((last - first) * (GData::thumbPagesReadahead + 1));
-		if (last >= thumbViewModel->rowCount())
-			last = thumbViewModel->rowCount() - 1;
-	}
-	else
-	{
-		first -= (last - first) * (GData::thumbPagesReadahead + 1);
-		if (first < 0)
-			first = 0;
-
-		last += 10;
-		if (last >= thumbViewModel->rowCount())
-			last = thumbViewModel->rowCount() - 1;
-	}
-
-	if (thumbsRangeFirst == first && thumbsRangeLast == last) {
+	int firstVisible = getFirstVisibleThumb();
+	int lastVisible = getLastVisibleThumb();
+	if (abortOp || firstVisible < 0 || lastVisible < 0) {
 		return;
 	}
 
-	thumbsRangeFirst = first;
-	thumbsRangeLast = last;
+	if (scrolledForward) {
+		lastVisible += ((lastVisible - firstVisible) * (GData::thumbPagesReadahead + 1));
+		if (lastVisible >= thumbViewModel->rowCount()) {
+			lastVisible = thumbViewModel->rowCount() - 1;
+		}
+	} else {
+		firstVisible -= (lastVisible - firstVisible) * (GData::thumbPagesReadahead + 1);
+		if (firstVisible < 0) {
+			firstVisible = 0;
+		}
+
+		lastVisible += 10;
+		if (lastVisible >= thumbViewModel->rowCount()) {
+			lastVisible = thumbViewModel->rowCount() - 1;
+		}
+	}
+
+	if (thumbsRangeFirst == firstVisible && thumbsRangeLast == lastVisible) {
+		return;
+	}
+
+	thumbsRangeFirst = firstVisible;
+	thumbsRangeLast = lastVisible;
 
 	loadThumbsRange();
-	if (!abortOp)
+	if (!abortOp) {
 		goto Start;
+	}
 }
 
 int ThumbView::getFirstVisibleThumb()
 {
 	QModelIndex idx;
 
-	for (int currThumb = 0; currThumb < thumbViewModel->rowCount(); ++currThumb)
-	{
+	for (int currThumb = 0; currThumb < thumbViewModel->rowCount(); ++currThumb) {
 		idx = thumbViewModel->indexFromItem(thumbViewModel->item(currThumb));
-		if (viewport()->rect().contains(QPoint(0, visualRect(idx).y() + visualRect(idx).height() + 1)))
-		{
+		if (viewport()->rect().contains(QPoint(0, visualRect(idx).y() + visualRect(idx).height() + 1))) {
 			return idx.row();
 		}
 	}
@@ -369,11 +436,9 @@ int ThumbView::getLastVisibleThumb()
 {
 	QModelIndex idx;
 
-	for (int currThumb = thumbViewModel->rowCount() -1; currThumb >= 0 ; --currThumb)
-	{
+	for (int currThumb = thumbViewModel->rowCount() -1; currThumb >= 0 ; --currThumb) {
 		idx = thumbViewModel->indexFromItem(thumbViewModel->item(currThumb));
-		if (viewport()->rect().contains(QPoint(0, visualRect(idx).y() + visualRect(idx).height() + 1)))
-		{
+		if (viewport()->rect().contains(QPoint(0, visualRect(idx).y() + visualRect(idx).height() + 1))) {
 			return idx.row();
 		}
 	}
@@ -383,8 +448,7 @@ int ThumbView::getLastVisibleThumb()
 
 bool ThumbView::isThumbVisible(QModelIndex idx)
 {
-	if (viewport()->rect().contains(QPoint(0, visualRect(idx).y() + visualRect(idx).height() + 1)))
-	{
+	if (viewport()->rect().contains(QPoint(0, visualRect(idx).y() + visualRect(idx).height() + 1))) {
 		return true;
 	}
 
@@ -395,37 +459,23 @@ void ThumbView::updateThumbsCount()
 {
 	QString state ;
 	
-	if (thumbViewModel->rowCount() > 0)
-	{
-		state = (QString::number(thumbViewModel->rowCount()) + tr(" images"));
+	if (thumbViewModel->rowCount() > 0) {
+		state = tr("%n image(s)", "", thumbViewModel->rowCount());
+	} else {
+		state = tr("No images");
 	}
-	else {
-		state = QString::number(thumbViewModel->rowCount()) + tr(" images");
-	}
-
+	thumbsDir->setPath(GData::currentViewDir);
 	emit setStatus(state);
 }
 
-void ThumbView::updateThumbsSelection()
-{
-	QString state;
-	int nSelected = selectionModel()->selectedIndexes().size();
-
-	if (!nSelected)
-		state = QString::number(thumbViewModel->rowCount()) + tr(" images");
-	else if (nSelected >= 1)
-		state = QString(tr("Selected ") + QString::number(nSelected) + tr(" of ")
-							+ QString::number(thumbViewModel->rowCount()) + tr(" images"));
-	emit setStatus(state);
-}
-
-void ThumbView::load()
+void ThumbView::loadPrepare()
 {
 	float thumbAspect = 1.33;
-	if (GData::thumbsLayout == Compact)
+	if (GData::thumbsLayout == Compact) {
 		thumbAspect = 1.77;
-	else if (GData::thumbsLayout == Squares)
+	} else if (GData::thumbsLayout == Squares) {
 		thumbAspect = 2;
+	}
 
 	thumbHeight = (GData::thumbsLayout == Squares)? thumbSize * thumbAspect : thumbSize;
 	thumbWidth = (GData::thumbsLayout == Squares)? thumbSize * thumbAspect : thumbHeight * thumbAspect;
@@ -434,38 +484,50 @@ void ThumbView::load()
 	fileFilters->clear();
 	QString textFilter("*");
 	textFilter+= filterStr;
-	*fileFilters	<< textFilter + "*.BMP"
-					<< textFilter + "*.GIF"
-					<< textFilter + "*.ICO"
-					<< textFilter + "*.JPEG"
-					<< textFilter + "*.JPG"
-					<< textFilter + "*.MNG"
-					<< textFilter + "*.PBM"
-					<< textFilter + "*.PGM"
-					<< textFilter + "*.PNG"
-					<< textFilter + "*.PPM"
-					<< textFilter + "*.SVG"
-					<< textFilter + "*.SVGZ"
-					<< textFilter + "*.TGA"					
-					<< textFilter + "*.TIF"
-					<< textFilter + "*.TIFF"
-					<< textFilter + "*.WBMP"
-					<< textFilter + "*.XBM"
-					<< textFilter + "*.XPM"
-					<< textFilter + "*.JPE";
+	*fileFilters	<< textFilter + "*.bmp"
+					<< textFilter + "*.cur"
+					<< textFilter + "*.dds"
+					<< textFilter + "*.gif"
+					<< textFilter + "*.icns"
+					<< textFilter + "*.ico"
+					<< textFilter + "*.jpeg"
+					<< textFilter + "*.jpg"
+					<< textFilter + "*.jp2"
+					<< textFilter + "*.jpe"
+					<< textFilter + "*.mng"
+					<< textFilter + "*.pbm"
+					<< textFilter + "*.pgm"
+					<< textFilter + "*.png"
+					<< textFilter + "*.ppm"
+					<< textFilter + "*.svg"
+					<< textFilter + "*.svgz"
+					<< textFilter + "*.tga"					
+					<< textFilter + "*.tif"
+					<< textFilter + "*.tiff"
+					<< textFilter + "*.wbmp"
+					<< textFilter + "*.webp"
+					<< textFilter + "*.xbm"
+					<< textFilter + "*.xpm";
 	thumbsDir->setNameFilters(*fileFilters);
 	thumbsDir->setFilter(QDir::Files);
-	if (GData::showHiddenFiles)
+	if (GData::showHiddenFiles) {
 		thumbsDir->setFilter(thumbsDir->filter() | QDir::Hidden);
+	}
 	
-	thumbsDir->setPath(currentViewDir);
-	thumbsDir->setSorting(thumbsSortFlags);
-	thumbViewModel->clear();
+	thumbsDir->setPath(GData::currentViewDir);
 
+	QDir::SortFlags tempThumbsSortFlags = thumbsSortFlags;
+	if (tempThumbsSortFlags & QDir::Size || tempThumbsSortFlags & QDir::Time) {
+		tempThumbsSortFlags ^= QDir::Reversed;
+	}
+	thumbsDir->setSorting(tempThumbsSortFlags);
+
+	thumbViewModel->clear();
 	setSpacing(GData::thumbSpacing);
 
-	if (isNeedScroll)
+	if (isNeedScroll) {
 		scrollToTop();
+	}
 
 	abortOp = false;
 	newIndex = 0;
@@ -473,32 +535,69 @@ void ThumbView::load()
 	thumbsRangeFirst = -1;
 	thumbsRangeLast = -1;
 
+	imageTags->resetTagsState();
+}
+
+void ThumbView::load()
+{
+	emit showBusy(true);
+	loadPrepare();
 	initThumbs();
 	updateThumbsCount();
 	loadVisibleThumbs();
 
-	if (GData::includeSubFolders)
-	{
-		emit showBusy(true);
-		QDirIterator iterator(currentViewDir, QDirIterator::Subdirectories);
-		while (iterator.hasNext())
-		{
+	if (GData::includeSubFolders) {
+		QDirIterator iterator(GData::currentViewDir, QDirIterator::Subdirectories);
+		while (iterator.hasNext()) {
 			iterator.next();
-			if (iterator.fileInfo().isDir() && iterator.fileName() != "." && iterator.fileName() != "..")
-			{
+			if (iterator.fileInfo().isDir() && iterator.fileName() != "." && iterator.fileName() != "..") {
 				thumbsDir->setPath(iterator.filePath());
+
 				initThumbs();
 				updateThumbsCount();
 				loadVisibleThumbs();
 
-				if (abortOp)
-				{
+				if (abortOp) {
+					goto finish;
+				}
+			}
+			QApplication::processEvents();
+		}
+
+		QItemSelection dummy;
+		handleSelectionChanged(dummy);
+	}
+
+finish:
+	emit showBusy(false);
+	busy = false;
+	return;
+}
+
+void ThumbView::loadDuplicates()
+{
+	loadPrepare();
+
+	emit showBusy(true);
+	emit setStatus(tr("Searching duplicate images..."));
+
+	dupImageHashes.clear();
+	findDupes(true);
+
+	if (GData::includeSubFolders) {
+		QDirIterator iterator(GData::currentViewDir, QDirIterator::Subdirectories);
+		while (iterator.hasNext()) {
+			iterator.next();
+			if (iterator.fileInfo().isDir() && iterator.fileName() != "." && iterator.fileName() != "..") {
+				thumbsDir->setPath(iterator.filePath());
+
+				findDupes(false);
+				if (abortOp) {
 					goto finish;
 				}
 			}
 			QApplication::processEvents();
 		}
-		updateThumbsSelection();
 	}
 
 finish:
@@ -511,35 +610,140 @@ void ThumbView::initThumbs()
 {
 	thumbFileInfoList = thumbsDir->entryInfoList();
 	static QStandardItem *thumbIitem;
-	static int currThumb;
+	static int fileIndex;
 	static QPixmap emptyPixMap;
 	static QSize hintSize;
+	int processEventsCounter = 0;
 
 	emptyPixMap = QPixmap::fromImage(emptyImg).scaled(thumbWidth, thumbHeight);
 
-	if (GData::thumbsLayout == Squares)
+	if (GData::thumbsLayout == Squares) {
 		hintSize = QSize(thumbWidth / 2, thumbWidth / 2);
-	else if (GData::thumbsLayout == Classic)
+	} else if (GData::thumbsLayout == Classic) {
 		hintSize = QSize(thumbWidth, thumbHeight + 
-							(GData::showLabels? QFontMetrics(font()).height() + 5 : 0));
+						(GData::showLabels? QFontMetrics(font()).height() + 5 : 0));
+	}
 
-	for (currThumb = 0; currThumb < thumbFileInfoList.size(); ++currThumb)
-	{
-		thumbFileInfo = thumbFileInfoList.at(currThumb);
+	for (fileIndex = 0; fileIndex < thumbFileInfoList.size(); ++fileIndex) {
+		thumbFileInfo = thumbFileInfoList.at(fileIndex);
+
+		mdCache->loadImageMetadata(thumbFileInfo.filePath());
+		if (imageTags->folderFilteringActive && imageTags->isImageFilteredOut(thumbFileInfo.filePath())) {
+			continue;
+		}
+		
 		thumbIitem = new QStandardItem();
 		thumbIitem->setData(false, LoadedRole);
-		thumbIitem->setData(currThumb, SortRole);
+		thumbIitem->setData(fileIndex, SortRole);
 		thumbIitem->setData(thumbFileInfo.filePath(), FileNameRole);
-		if (GData::thumbsLayout != Squares && GData::showLabels)
+
+		if (GData::thumbsLayout != Squares && GData::showLabels) {
 			thumbIitem->setData(thumbFileInfo.fileName(), Qt::DisplayRole);
-		if (GData::thumbsLayout == Compact)
+		}
+
+		if (GData::thumbsLayout == Compact) {
 			thumbIitem->setIcon(emptyPixMap);
+		}
+
 		thumbIitem->setTextAlignment(Qt::AlignTop | Qt::AlignHCenter);
-		if (GData::thumbsLayout != Compact)
+		if (GData::thumbsLayout != Compact) {
 			thumbIitem->setSizeHint(hintSize);
+		}
 
 		thumbViewModel->appendRow(thumbIitem);
+
+		++processEventsCounter;
+		if (processEventsCounter > 100) {
+			processEventsCounter = 0;
+			QApplication::processEvents();
+		}
 	}
+
+    imageTags->populateTagsTree();
+	if (imageTags->currentDisplayMode == SelectionTagsDisplay) {
+		imageTags->showSelectedImagesTags();
+	} else {
+		imageTags->showTagsFilter();
+	}
+
+	if (thumbFileInfoList.size() && selectionModel()->selectedIndexes().size() == 0) {
+		selectThumbByRow(0);
+	}
+}
+
+void ThumbView::selectThumbByRow(int row)
+{
+	setCurrentIndexByRow(row);
+	selectCurrentIndex();
+}
+
+void ThumbView::updateFoundDupesState(int duplicates, int filesScanned, int originalImages)
+{
+	QString state;
+	state = tr("Scanned %1, displaying %2 (%3 and %4)")
+				.arg(tr("%n image(s)", "", filesScanned))
+				.arg(tr("%n image(s)", "", originalImages + duplicates))
+				.arg(tr("%n original(s)", "", originalImages))
+				.arg(tr("%n duplicate(s)", "", duplicates));
+	emit setStatus(state);
+}
+
+void ThumbView::findDupes(bool resetCounters)
+{
+	thumbFileInfoList = thumbsDir->entryInfoList();
+	int processEventsCounter = 0;
+	static int originalImages;
+	static int foundDups;
+	static int totalFiles;
+	if (resetCounters) {
+		originalImages = totalFiles = foundDups = 0;
+	}
+
+	for (int currThumb = 0; currThumb < thumbFileInfoList.size(); ++currThumb) {
+		thumbFileInfo = thumbFileInfoList.at(currThumb);
+	    QCryptographicHash md5gen(QCryptographicHash::Md5);
+	    QString currentFilePath = thumbFileInfo.filePath();
+
+	    QFile file(currentFilePath);
+		if (!file.open(QIODevice::ReadOnly)) {
+		    continue;
+		}
+		totalFiles++;
+
+	    md5gen.addData(file.readAll());
+	    file.close();
+		QString md5 = md5gen.result().toHex();
+
+		if (dupImageHashes.contains(md5)) {
+			if (dupImageHashes[md5].duplicates < 1) {
+				addThumb(dupImageHashes[md5].filePath);
+				originalImages++;
+			}
+
+			foundDups++;
+			dupImageHashes[md5].duplicates++;
+			addThumb(currentFilePath);
+		} else {
+			DuplicateImage dupImage;
+			dupImage.filePath = currentFilePath; 
+			dupImage.duplicates = 0; 
+			dupImageHashes.insert(md5, dupImage);
+		}
+		
+		++processEventsCounter;
+		if (processEventsCounter > 9) {
+			processEventsCounter = 0;
+			QApplication::processEvents();
+		}
+		
+		updateFoundDupesState(foundDups, totalFiles, originalImages);
+		
+		if (abortOp) {
+			break;
+		}
+	}
+
+	updateFoundDupesState(foundDups, totalFiles, originalImages);
 }
 
 void ThumbView::loadThumbsRange()
@@ -549,10 +753,11 @@ void ThumbView::loadThumbsRange()
 	static QSize currThumbSize;
 	static int currRowCount;
 	static QString imageFileName;
-	static QImage thumb;
+	QImage thumb;
 	int currThumb;
+	bool imageReadOk;
 
-	if (inProgress) {	
+	if (inProgress) {
 		abortOp = true;
 		QTimer::singleShot(0, this, SLOT(loadThumbsRange()));
 		return;
@@ -563,20 +768,22 @@ void ThumbView::loadThumbsRange()
 
 	for (	scrolledForward? currThumb = thumbsRangeFirst : currThumb = thumbsRangeLast;
 			(scrolledForward? currThumb <= thumbsRangeLast : currThumb >= thumbsRangeFirst);
-			scrolledForward? ++currThumb : --currThumb)
-	{
-		if (abortOp || thumbViewModel->rowCount() != currRowCount)
-			break;
+			scrolledForward? ++currThumb : --currThumb) {
 
-		if (thumbViewModel->item(currThumb)->data(LoadedRole).toBool())
+		if (abortOp || thumbViewModel->rowCount() != currRowCount || currThumb < 0) {
+			break;
+		}
+
+		if (thumbViewModel->item(currThumb)->data(LoadedRole).toBool()) {
 			continue;
+		}
 
 		imageFileName = thumbViewModel->item(currThumb)->data(FileNameRole).toString();
 		thumbReader.setFileName(imageFileName);
 		currThumbSize = thumbReader.size();
+		imageReadOk = false;
 
-		if (currThumbSize.isValid())
-		{
+		if (currThumbSize.isValid()) {
 			if (!GData::noEnlargeSmallThumb
 				|| (currThumbSize.width() > thumbWidth || currThumbSize.height() > thumbHeight))
 			{
@@ -584,27 +791,25 @@ void ThumbView::loadThumbsRange()
 			}
 
 			thumbReader.setScaledSize(currThumbSize);
-			thumb = thumbReader.read();
+			imageReadOk = thumbReader.read(&thumb);
+		}
 
-			if (GData::exifThumbRotationEnabled)
-			{
-				ImageView::rotateByExifRotation(thumb, imageFileName);
+		if (imageReadOk) {
+			if (GData::exifThumbRotationEnabled) {
+				imageView->rotateByExifRotation(thumb, imageFileName);
 				currThumbSize = thumb.size();
 				currThumbSize.scale(QSize(thumbWidth, thumbHeight), Qt::KeepAspectRatio);
 			}
-				
+			
 			thumbViewModel->item(currThumb)->setIcon(QPixmap::fromImage(thumb));
-		}
-		else
-		{
+		} else {
 			thumbViewModel->item(currThumb)->setIcon(QIcon::fromTheme("image-missing",
 									QIcon(":/images/error_image.png")).pixmap(BAD_IMG_SZ, BAD_IMG_SZ));
 			currThumbSize.setHeight(BAD_IMG_SZ);
 			currThumbSize.setWidth(BAD_IMG_SZ);
 		}
 
-		if (GData::thumbsLayout == Compact)
-		{
+		if (GData::thumbsLayout == Compact) {
 			if (GData::showLabels)
 				currThumbSize.setHeight(currThumbSize.height() + QFontMetrics(font()).height() + 5);
 			thumbViewModel->item(currThumb)->setSizeHint(currThumbSize);
@@ -617,8 +822,7 @@ void ThumbView::loadThumbsRange()
 		QApplication::processEvents();
 	}
 
-	if (GData::thumbsLayout == Compact && thumbViewModel->rowCount() > 0)
-	{
+	if (GData::thumbsLayout == Compact && thumbViewModel->rowCount() > 0) {
 		setRowHidden(0 , false);
 	}
 
@@ -662,7 +866,7 @@ void ThumbView::addThumb(QString &imageFullPath)
 
 		if (GData::exifThumbRotationEnabled)
 		{
-			ImageView::rotateByExifRotation(thumb, imageFullPath);
+			imageView->rotateByExifRotation(thumb, imageFullPath);
 			currThumbSize = thumb.size();
 			currThumbSize.scale(QSize(thumbWidth, thumbHeight), Qt::KeepAspectRatio);
 		}
@@ -717,48 +921,13 @@ void ThumbView::invertSelection()
 	selectionModel()->select(toggleSelection, QItemSelectionModel::Toggle);
 }
 
-FSTree::FSTree(QWidget *parent) : QTreeView(parent)
+void ThumbView::setNeedScroll(bool needScroll)
 {
-	setAcceptDrops(true);
-	setDragEnabled(true);
-	setDragDropMode(QAbstractItemView::InternalMove);
-
-	connect(this, SIGNAL(expanded(const QModelIndex &)),
-								this, SLOT(resizeTreeColumn(const QModelIndex &)));
-	connect(this, SIGNAL(collapsed(const QModelIndex &)),
-								this, SLOT(resizeTreeColumn(const QModelIndex &)));
+	isNeedScroll = needScroll;
 }
 
-QModelIndex FSTree::getCurrentIndex()
+void ThumbView::setImageView(ImageView *imageView)
 {
-	return selectedIndexes().first();
-}
-
-void FSTree::resizeTreeColumn(const QModelIndex &)
-{
-	resizeColumnToContents(0);
-}
-
-void FSTree::dragEnterEvent(QDragEnterEvent *event)
-{
-	QModelIndexList selectedDirs = selectionModel()->selectedRows();
-	dndOrigSelection = selectedDirs[0];
-	event->acceptProposedAction();
-}
-
-void FSTree::dragMoveEvent(QDragMoveEvent *event)
-{
-	setCurrentIndex(indexAt(event->pos()));
-}
-
-void FSTree::dropEvent(QDropEvent *event)
-{
-	if (event->source())
-	{
-		QString fstreeStr="FSTree";
-		bool dirOp = (event->source()->metaObject()->className() == fstreeStr);
-		emit dropOp(event->keyboardModifiers(), dirOp, event->mimeData()->urls().at(0).toLocalFile());
-		setCurrentIndex(dndOrigSelection);
-	}
+	this->imageView  = imageView;
 }
 
